# Functions that are exported when MXP is running.
# These functions are used internally by MXP scripts and may be used by parameter / method scripts.

# Functions to analyze exit status of the last command. {{{
# Return the rightmost non-0 exit status of a command in the preceeding pipe. {{{
# If the last executed command is a pipe (like "find . -type d | wc -l"), the value of '$?'
# variable is the exit status of last command in the pipe (i.e., "wc -l"), while any command
# in the pipe may fail. This function returns non-0 exit status if any command in a pipe fails.
# This behavior is similar to the one provided by bash with "set -o pipefail", but address
# only the last executed pipe.
pipefail_status () {
    local ps=(${PIPESTATUS[@]})
    local i
    for ((i=${#ps[@]}-1; i>=0; i--)); do
        if ((${ps[$i]} > 0)); then return ${ps[$i]}; fi
    done
    return 0
}
export -f pipefail_status
#}}}

# Exit from the current shell if previous command procuded error. {{{
# An optional argument is error message, which is printed to stderr if error is detected.
# The important problem that this function has to solve is to stop further execution of a script
# if an error was detected. This is achieved by sending SIGTERM to the MXP process and all its
# children -- see discussion in comments to 'mxp1' script.
# If previous command was a pipe, and any error in the pipe should be considered as failure,
# use: "pipefail_status; exit_if_error"
exit_if_error () {
    local rc=$?
    if (($rc == 0)); then return 0; fi
    if [ -n "$1" ]; then (>&2 echo "$1"); fi
    # Find pids of all descendants. Note: some pids are not valid when this function exits.
    list_descendants () {
        echo $1
        local children=$(ps -o pid= --ppid $1)
        local pid; for pid in $children; do list_descendants $pid; done
    }
    local descendants=$(list_descendants $MXP_WORKER_PID)
    kill -s SIGTERM $descendants &> /dev/null
    # We should not be at this point!
}
export -f exit_if_error
#}}}
#}}}

# Record script: copy file to $MXP_TARGET_DIR/.mxp/tmp and print its path to stdout. {{{
# Parameter: name of script to record (filename only, MUST be in the PATH).
# If script file is not found, exits the whole mxp process tree (we should die anyway...)
# Usages:
#   1) bash
#       source $(mxp_locate_and_record_script SCRIPT_NAME)
#   2) R
#       source(system('bash -c "mxp_locate_and_record_script RSCRIPT_NAME.R"', intern=T))
#   3) Python
#       import subprocess
#       .....
#       subprocess.check_output(["bash", "-c", "mxp_locate_and_record_script PYM.py"])
#       import PYM
#     (note: as all subdirectories are added to PYTHONPATH, nothing special is needed to import
#     module; mxp_locate_and_record_script is used only to record script in .mxp directory.)
mxp_locate_and_record_script () {
    # Note: "local x=$(cmd)" produces $?==0
    local script=$1  script_file=""
    script_file=$(type -P $script 2> /dev/null)
    exit_if_error "====  [ERR]  Script '$script' is not found in the PATH."
    if [ ! -f $MXP_TARGET_DIR/.mxp/tmp/$script ]; then
        cp  $script_file  $MXP_TARGET_DIR/.mxp/tmp/$script
        if ((MXP_VERBOSITY > 0)); then (>&2 echo "====  Using script:  $script_file"); fi
    fi
    echo $script_file
}
export -f mxp_locate_and_record_script
#}}}
